
/*
hxl (c) 2010-always Jan Panteltje

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/



#include "hxl.h"


typedef unsigned int  word;
typedef unsigned char byte, bit;


// *********************************************************************
// HEX FILE LOADER
// *********************************************************************

// This is for Intel INHX8M (8-bit merged) hex files only.
// These files use two bytes for each word (low, then high).
// All addresses are doubled, i.e., 0x2001 is encoded as 0x4002,
// so that addresses increment at the same rate as the byte count.

int valid_hex_line(char *s)				// Gross syntax and checksum check.
{
byte cksum = 0;                     	// For all HEX formats, not just 8M.
int bytecount;
int i, b;

if (s[0] != ':') return(0);          	// Initial colon

sscanf(s + 1, "%2x", &bytecount);
if (bytecount > 32) return(0);       	// Valid byte count

cksum = bytecount;
i = 3;
bytecount = bytecount + 3;

while (bytecount > 0)
	{
	bytecount--;
	sscanf(s + i, "%2x", &b);
	cksum = cksum + b;                  // Compute checksum
	i = i + 2;
	}

sscanf(s + i, "%2x", &b);

cksum =- cksum;
if (cksum == b) return 1;            	// Test checksum

return 0;
} /* end function valid_hex_line */


// Loads a hex file into memory arrays
int load_hex_file(FILE *f, int *length, int *buffer_top, int *min_address)
{
char s[256];
word i;
word linetype = 0;   // 0 for data, 1 for end of file
word wordcount;      // number of 16 bit words on this line
word address;        // address where they begin
int max_address;
int line;
int position;
unsigned int ua;

if(debug_flag)
	{	
	fprintf(stderr, "hxl: load_hex_file(): arg *f=%p *length=%p *buffer_top=%p *min_address=%p\n", f, length, buffer_top, min_address);
	}

*length = 0;
*min_address = 0xffff;
max_address = 0;
line = 0;
while((!feof(f)) && (linetype != 1))
	{
    fgets(s, 255, f);
	line++;

	if(!valid_hex_line(s)) 
		{			// Syntax check
		s[80] = 0;	// Truncate invalid line for display  was 40

		if (s[0] != ':')
			{
       		fprintf(stderr, "hxl: invalid line (skipped): '%s'...\n", s);
       	 	continue;
     		}
		else
			{
    	    fprintf(stderr, "hxl: unable to decode line:  '%s'...\n", s);


			return 0;
			}
		}

	sscanf(s + 1, "%2x", &wordcount);      // Parse the line - Intel Hex8M

	sscanf(s + 3, "%4x", &address);

	sscanf(s + 7, "%2x", &linetype);

	if(linetype == 1)
		{
		(*buffer_top) = max_address;

		return 1;
		}

	if(address < *min_address) *min_address = address;

	if(debug_flag)
		{
		fprintf(stderr, "hxl: hex line address=0x%04x (%d)\n", address, address);
		}
		
	/* grab the data */
	for (i = 0; i < wordcount; i++)
		{
		position = address + i;
		sscanf(s + (9 + (2 * i) ), "%2x", &ua);

		if(buffer_flags[position] != -1)
			{
			fprintf(stderr, "hxl: trying to store data 0x%02x (%d) (%c), but address 0x%04x (%d) is already occupied with data 0x%02x (%d) (%c)\n",\
			ua, ua, ua, position, position, buffer[position], buffer[position], buffer[position]);							
		
			return 0;
			}	

		buffer[position] = (unsigned char)ua;

		(*length)++;

		if(position > max_address) max_address = position;

		if(debug_flag)
			{
			fprintf(stderr, "hxl: line %d=%s\n", line, s); 
			fprintf(stderr, "hxl: line %d  address=0x%04x (%d)  byte=0x%02x (%d)   data=0x%02x (%d) (%c)\n", line, position, position, *length, *length, buffer[position], buffer[position], buffer[position]);
			}
			
		} /* end for i */

	} /* end while data */


/* never here */
return 1;
} /* end function load_hex_file */

