

/*
hxl (c) 2010-always Jan Panteltje

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "hxl.h"



char *strsave(char *s) /*save char array s somewhere*/
{
char *p;

p = malloc(strlen(s) +  1);
if(p)  strcpy(p, s);
return p;
}



void print_usage()
{
fprintf(stderr,
"\nPanteltje hxl-%s\n\
Usage:\nhxl [-b basic_file_out] [-c bin_file_out] [-h] [-i intel_hex_file_in] [-l data_bytes_in_a_DATA_line] [-v]\n\
-b filename   MCS-51 BASIC output filename, writes code as DATA statements, the first DATA statement's line number is the same as the org address.\n\
-c filename   output file, binary image of buffer.\n\
-h            help (this help).\n\
-i filename   Intel hex format file in.\n\
-l int        data bytes in a DATA statement per BASIC line, default 10.\n\
-v            verbose.\n\
\n",\
PROG_VERSION);

fprintf(stderr, "Example:\n\
hxl -i file.hex -b file.bas -l 10\n"\
);

} /* end function print_usage */



int main(int argc, char **argv)
{
int a, i, j;
FILE *ifptr = 0;
FILE *ofptr = 0;
FILE *bfptr = 0;
char *ifilename = 0;
char *ofilename = 0;
char *bfilename = 0;
int length = 0;
int min_address = 0;
int buffer_top;
int basic_line;
char temp[512];
int data_per_line;


/* defaults */
debug_flag = 0;
data_per_line = 10;
/* end defaults */

while(1)
	{
	a = getopt(argc, argv, "c:b:hi:l:v");
	if(a == -1) break;

	switch(a)
		{
		case 'b': // output filename code in MSC51 BASIC  data statements
			bfilename = strsave(optarg);
			if(! bfilename)
				{
				fprintf(stderr, "hxl: could not allocate space for bfilename, aborting.\n");

				exit(1);
				}	
				
			break;
		case 'c': // output binary dump of code
			ofilename = strsave(optarg);
			if(! ofilename)
				{
				fprintf(stderr, "hxl: could not allocate space for ofilename, aborting.\n");

				exit(1);
				}	
			break;		
		case 'h': // help
			print_usage();
			exit(1);
			break;
		case 'i': // input Intel hex filename
			ifilename = strsave(optarg);
			if(! ifilename)
				{
				fprintf(stderr, "hxl: could not allocate space for ifilename, aborting.\n");

				exit(1);
				}	
				
			break;
		case 'v': // debug mode
			debug_flag = 1;
			break;
		case 'l': //  DATA bytes per BASIC DATA line 
			data_per_line = atoi(optarg);
			if(data_per_line <= 0)
				{
				fprintf(stderr, "hxl: -l flag, need at least 1 data byte to follow a DATA statement, aborting.\n");

				exit(1);
				}
			break;
        case -1:
        	break;
		case '?':
			if (isprint(optopt) )
 				{
 				fprintf(stderr, "hxl: unknown option `-%c'.\n", optopt);
 				}
			else
				{
				fprintf(stderr, "hxl: unknown option character `\\x%x'.\n", optopt);
				}
			print_usage();

			exit(1);
			break;			
		default:
			print_usage();

			exit(1);
			break;
		}/* end switch a */
	}/* end while getopt() */


if(! ifilename)
	{
	fprintf(stderr, "hxl: no input filename specified, try hxl -h for help, aborting.\n");

	exit(1);
	}
	

ifptr = fopen(ifilename, "r");
if(! ifptr)
	{
	fprintf(stderr, "hxl: could not open file %s for read, aborting.\n", ifilename);

	exit(1);
	}

if(ofilename)
	{
	ofptr = fopen(ofilename, "w");
	if(! ofptr)
		{
		fprintf(stderr, "hxl: could not open file %s for write, aborting.\n", ofilename);

		exit(1);
		}
	}
	
if(bfilename)
	{
	strcpy(temp, bfilename);
	if(! strchr(temp, '.') ) strcat(temp, ".bas");
	bfptr = fopen(temp, "w");
	if(! bfptr)
		{
		fprintf(stderr, "hxl: could not open file %s for write, aborting.\n", temp);

		exit(1);
		}
	}

buffer = (unsigned char*)malloc(65536);
if(! buffer)
	{
	fprintf(stderr, "hx;: could not allocate space for buffer, aborting.\n"); 
	
	exit(1);
	}

buffer_flags = (char *)malloc(65536);
if(! buffer_flags)
	{
	fprintf(stderr, "hxl: could not allocate space for buffer_flags, aborting.\n"); 
	
	exit(1);
	}

/* clear buffer */
for(i = 0l; i < 65535; i++) buffer[i] = 0;

/* clear buffer_flags */
for(i = 0; i < 65535; i++) buffer_flags[i] = -1; /* indicates not used */
	
//int load_hex_file(FILE *f, int *length, int *buffer_top, int *min_address);
if(! load_hex_file(ifptr, &length, &buffer_top, &min_address) )
	{
	fprintf(stderr, "hxl: could not load hex file, aborting.\n");
	
	exit(1);
	}

fprintf(stderr, "hxl: %d bytes loaded, min_address=0x%04x (%d) buffer_top=0x%04x (%d)\n", length, min_address, min_address, buffer_top, buffer_top);

/* write binary output file */
if(ofptr)
	{
	for(i = min_address; i <= buffer_top; i++)
		{
		fprintf(ofptr, "%c", buffer[i]);

		if(debug_flag)
			{
			fprintf(stderr, "hxl: buffer: address=0x%04x (%d)  data=0x%02x (%d) (%c)\n", i, i, buffer[i], buffer[i], buffer[i]);
			}
		}

	fclose(ofptr);	
	}

if(debug_flag)
	{
	for(i = min_address; i < buffer_top; i++) fprintf(stdout, "0x%04x (%d)  0x%02x (%d) (%c)  flags=%d\n", i, i, buffer[i], buffer[i], buffer[i], buffer_flags[i]);
	}

if(bfptr)
	{
	basic_line = min_address - 2;

	fprintf(bfptr, "%d REM code from %s  by hxl from %s\n", basic_line , temp, ifilename);
	basic_line++;
	
	fprintf(bfptr, "%d REM org=%d last address used=%d\n", basic_line, min_address, buffer_top);
	basic_line++;
	
	i = min_address;
	while(1)
		{
		fprintf(bfptr, "%d DATA ", basic_line);
		basic_line++;
		
		for(j = 0; j < data_per_line; j++)
			{
			fprintf(bfptr, "%d", buffer[i]);
			i++;
			if(i > buffer_top) break; 

			if(j < data_per_line - 1) fprintf(bfptr, ", ");
			} /* end for all data bytes in a  BASIC DATA line */

		fprintf(bfptr, "\n");

		if(i > buffer_top) break;
		} /* end while all addresses */
		
	fprintf(bfptr, "%d FOR I=%d TO %d\n",  basic_line, min_address, buffer_top);
	basic_line++;

	fprintf(bfptr, "%d READ A : XBY(I)=A : NEXT\n", basic_line);
	basic_line++;

	fprintf(bfptr, "%d RETURN\n", basic_line);

	fclose(bfptr);
	}

exit(0);
} /* end function main */

